unit Keypad3x4;

{ ********************************************************************************************************
    Abfrage einer Zehnertastatur mit zwei Sondertasten (7-pol. Anschluss)
    ber einen I2C-Portexpander (PCF8574)
    J. Rathlev, Nov. 2006

    Interrupt: Int0 bei Tastenbettigung

    Anschlsse:
        Pin  E-Port     
         1     6      
         2     5
         3     4
         4     3           
         5     2
         6     1
         7     0           
        
    Abfrage:            
    Wert    Tastatur-Pins
      1        1-4       
      2        2-4       
      3        3-4       
      4        1-5       
      5        2-5       
      6        3-5       
      7        1-6       
      8        2-6       
      9        3-6       
      *        1-7       
      0        2-7       
      #        3-7

    E-Port 0,1,2,3  ==> Eingang
           4,5,6    ==> Ausgang
    }

interface

procedure InitKeypad (SubAdr : byte);

function KeypadPressed : boolean;

function ReadKeypad : char;
  
procedure Ext0Int; Interrupt External0; 

implementation

uses I2C, Math, Utils;

const
  KeyTab : string[12] = '147*2580369#';
var
  KeyFlag : boolean; volatile;
  KeyInp1,KeyInp2,
  KeyAdr,KeyOut  : byte; 

procedure Ext0Int; 
begin
  P1.7:=not P1.7;
  KeyFlag:=true;
  end;

procedure InitKeypad (SubAdr : byte);
begin
  KeyAdr:=SubAdr;
  KeyFlag:=false;
  KeyOut:=$0F;      // Bits 4,5,6 = 0 - 0,1,2,3 = input 
  I2CWriteDevice(PortExpander0,KeyAdr,1,Addr(KeyOut));
  EX0:=true;
  end;

function KeypadPressed : boolean;
begin
  Result:=KeyFlag;
  end;
  
function GetBit (AByte,ACount : byte) : byte;
begin
  while AByte and 1 <>0 do begin
    AByte:=AByte shr 1;
    dec(ACount);
    end;
  Result:=ACount-1;
  end; 
   
function ReadKeyPad : char;
begin
  repeat until KeyFlag;
  if I2CReadDevice(PortExpander0,KeyAdr,1,Addr(KeyInp1)) and (KeyInp1<>$F) then begin // read Bits 0,1,2,3
    KeyOut:=$70;      // Bits 0,1,2,3 = 0 - 4,5,6 = input
    I2CWriteDevice(PortExpander0,KeyAdr,1,Addr(KeyOut));
    I2CReadDevice(PortExpander0,KeyAdr,1,Addr(KeyInp2));   // read Bits 4,5,6
    KeyOut:=$0F;      // Bits 4,5,6 = 0 - 0,1,2,3 = input 
    I2CWriteDevice(PortExpander0,KeyAdr,1,Addr(KeyOut));
    Result:=KeyTab[4*GetBit(SwapNibble(KeyInp2),3)+GetBit(KeyInp1,4)+1];
    end
  else Result:=#0;
  LongDelay(100);  // wait for bouncing
  KeyFlag:=false;
  P1.0:=not P1.0;
  end;
  
end.